@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -29,9 +29,6 @@
 #endif
 #if defined(__OS2__)
 #include "core/os2/SDL_os2.h"
-#if SDL_THREAD_OS2
-#include "thread/os2/SDL_systls_c.h"
-#endif
 #endif
 
 /* this checks for HAVE_DBUS_DBUS_H internally. */
@@ -52,12 +49,13 @@
 #include "haptic/SDL_haptic_c.h"
 #include "joystick/SDL_joystick_c.h"
 #include "sensor/SDL_sensor_c.h"
+#include "thread/SDL_thread_c.h"
 
 /* Initialization/Cleanup routines */
-#if !SDL_TIMERS_DISABLED
+#ifndef SDL_TIMERS_DISABLED
 #include "timer/SDL_timer_c.h"
 #endif
-#if SDL_VIDEO_DRIVER_WINDOWS
+#ifdef SDL_VIDEO_DRIVER_WINDOWS
 extern int SDL_HelperWindowCreate(void);
 extern int SDL_HelperWindowDestroy(void);
 #endif
@@ -116,6 +114,7 @@ static SDL_bool SDL_MainIsReady = SDL_FALSE;
 #else
 static SDL_bool SDL_MainIsReady = SDL_TRUE;
 #endif
+static SDL_bool SDL_main_thread_initialized = SDL_FALSE;
 static SDL_bool SDL_bInMainQuit = SDL_FALSE;
 static Uint8 SDL_SubsystemRefCount[32];
 
@@ -160,11 +159,57 @@ static SDL_bool SDL_PrivateShouldQuitSubsystem(Uint32 subsystem)
     return (((subsystem_index >= 0) && (SDL_SubsystemRefCount[subsystem_index] == 1)) || SDL_bInMainQuit) ? SDL_TRUE : SDL_FALSE;
 }
 
+/* Private helper to either increment's existing ref counter,
+ * or fully init a new subsystem. */
+static SDL_bool SDL_PrivateInitOrIncrSubsystem(Uint32 subsystem)
+{
+    int subsystem_index = SDL_MostSignificantBitIndex32(subsystem);
+    SDL_assert((subsystem_index < 0) || (SDL_SubsystemRefCount[subsystem_index] < 255));
+    if (subsystem_index < 0) {
+        return SDL_FALSE;
+    }
+    if (SDL_SubsystemRefCount[subsystem_index] > 0) {
+        ++SDL_SubsystemRefCount[subsystem_index];
+        return SDL_TRUE;
+    }
+    return SDL_InitSubSystem(subsystem) == 0;
+}
+
 void SDL_SetMainReady(void)
 {
     SDL_MainIsReady = SDL_TRUE;
 }
 
+void SDL_InitMainThread(void)
+{
+    if (SDL_main_thread_initialized) {
+        return;
+    }
+
+    SDL_InitTLSData();
+#ifndef SDL_TIMERS_DISABLED
+    SDL_TicksInit();
+#endif
+    SDL_LogInit();
+
+    SDL_main_thread_initialized = SDL_TRUE;
+}
+
+static void SDL_QuitMainThread(void)
+{
+    if (!SDL_main_thread_initialized) {
+        return;
+    }
+
+    SDL_LogQuit();
+#ifndef SDL_TIMERS_DISABLED
+    SDL_TicksQuit();
+#endif
+    SDL_QuitTLSData();
+
+    SDL_main_thread_initialized = SDL_FALSE;
+}
+
 int SDL_InitSubSystem(Uint32 flags)
 {
     Uint32 flags_initialized = 0;
@@ -173,30 +218,14 @@ int SDL_InitSubSystem(Uint32 flags)
         return SDL_SetError("Application didn't initialize properly, did you include SDL_main.h in the file containing your main() function?");
     }
 
-    SDL_LogInit();
-
     /* Clear the error message */
     SDL_ClearError();
 
-#if SDL_USE_LIBDBUS
+#ifdef SDL_USE_LIBDBUS
     SDL_DBus_Init();
 #endif
 
-    if (flags & SDL_INIT_GAMECONTROLLER) {
-        /* game controller implies joystick */
-        flags |= SDL_INIT_JOYSTICK;
-    }
-
-    if (flags & (SDL_INIT_VIDEO | SDL_INIT_JOYSTICK | SDL_INIT_AUDIO)) {
-        /* video or joystick or audio implies events */
-        flags |= SDL_INIT_EVENTS;
-    }
-
-#if SDL_THREAD_OS2
-    SDL_OS2TLSAlloc(); /* thread/os2/SDL_systls.c */
-#endif
-
-#if SDL_VIDEO_DRIVER_WINDOWS
+#ifdef SDL_VIDEO_DRIVER_WINDOWS
     if (flags & (SDL_INIT_HAPTIC | SDL_INIT_JOYSTICK)) {
         if (SDL_HelperWindowCreate() < 0) {
             goto quit_and_error;
@@ -204,13 +233,9 @@ int SDL_InitSubSystem(Uint32 flags)
     }
 #endif
 
-#if !SDL_TIMERS_DISABLED
-    SDL_TicksInit();
-#endif
-
     /* Initialize the event subsystem */
     if (flags & SDL_INIT_EVENTS) {
-#if !SDL_EVENTS_DISABLED
+#ifndef SDL_EVENTS_DISABLED
         if (SDL_PrivateShouldInitSubsystem(SDL_INIT_EVENTS)) {
             if (SDL_EventsInit() < 0) {
                 goto quit_and_error;
@@ -226,7 +251,7 @@ int SDL_InitSubSystem(Uint32 flags)
 
     /* Initialize the timer subsystem */
     if (flags & SDL_INIT_TIMER) {
-#if !SDL_TIMERS_DISABLED && !SDL_TIMER_DUMMY
+#if !defined(SDL_TIMERS_DISABLED) && !defined(SDL_TIMER_DUMMY)
         if (SDL_PrivateShouldInitSubsystem(SDL_INIT_TIMER)) {
             if (SDL_TimerInit() < 0) {
                 goto quit_and_error;
@@ -242,8 +267,13 @@ int SDL_InitSubSystem(Uint32 flags)
 
     /* Initialize the video subsystem */
     if (flags & SDL_INIT_VIDEO) {
-#if !SDL_VIDEO_DISABLED
+#ifndef SDL_VIDEO_DISABLED
         if (SDL_PrivateShouldInitSubsystem(SDL_INIT_VIDEO)) {
+            /* video implies events */
+            if (!SDL_PrivateInitOrIncrSubsystem(SDL_INIT_EVENTS)) {
+                goto quit_and_error;
+            }
+
             if (SDL_VideoInit(NULL) < 0) {
                 goto quit_and_error;
             }
@@ -258,8 +288,13 @@ int SDL_InitSubSystem(Uint32 flags)
 
     /* Initialize the audio subsystem */
     if (flags & SDL_INIT_AUDIO) {
-#if !SDL_AUDIO_DISABLED
+#ifndef SDL_AUDIO_DISABLED
         if (SDL_PrivateShouldInitSubsystem(SDL_INIT_AUDIO)) {
+            /* audio implies events */
+            if (!SDL_PrivateInitOrIncrSubsystem(SDL_INIT_EVENTS)) {
+                goto quit_and_error;
+            }
+
             if (SDL_AudioInit(NULL) < 0) {
                 goto quit_and_error;
             }
@@ -274,8 +309,13 @@ int SDL_InitSubSystem(Uint32 flags)
 
     /* Initialize the joystick subsystem */
     if (flags & SDL_INIT_JOYSTICK) {
-#if !SDL_JOYSTICK_DISABLED
+#ifndef SDL_JOYSTICK_DISABLED
         if (SDL_PrivateShouldInitSubsystem(SDL_INIT_JOYSTICK)) {
+            /* joystick implies events */
+            if (!SDL_PrivateInitOrIncrSubsystem(SDL_INIT_EVENTS)) {
+                goto quit_and_error;
+            }
+
             if (SDL_JoystickInit() < 0) {
                 goto quit_and_error;
             }
@@ -289,8 +329,13 @@ int SDL_InitSubSystem(Uint32 flags)
     }
 
     if (flags & SDL_INIT_GAMECONTROLLER) {
-#if !SDL_JOYSTICK_DISABLED
+#ifndef SDL_JOYSTICK_DISABLED
         if (SDL_PrivateShouldInitSubsystem(SDL_INIT_GAMECONTROLLER)) {
+            /* game controller implies joystick */
+            if (!SDL_PrivateInitOrIncrSubsystem(SDL_INIT_JOYSTICK)) {
+                goto quit_and_error;
+            }
+
             if (SDL_GameControllerInit() < 0) {
                 goto quit_and_error;
             }
@@ -305,7 +350,7 @@ int SDL_InitSubSystem(Uint32 flags)
 
     /* Initialize the haptic subsystem */
     if (flags & SDL_INIT_HAPTIC) {
-#if !SDL_HAPTIC_DISABLED
+#ifndef SDL_HAPTIC_DISABLED
         if (SDL_PrivateShouldInitSubsystem(SDL_INIT_HAPTIC)) {
             if (SDL_HapticInit() < 0) {
                 goto quit_and_error;
@@ -321,7 +366,7 @@ int SDL_InitSubSystem(Uint32 flags)
 
     /* Initialize the sensor subsystem */
     if (flags & SDL_INIT_SENSOR) {
-#if !SDL_SENSOR_DISABLED
+#ifndef SDL_SENSOR_DISABLED
         if (SDL_PrivateShouldInitSubsystem(SDL_INIT_SENSOR)) {
             if (SDL_SensorInit() < 0) {
                 goto quit_and_error;
@@ -352,14 +397,11 @@ int SDL_Init(Uint32 flags)
 void SDL_QuitSubSystem(Uint32 flags)
 {
 #if defined(__OS2__)
-#if SDL_THREAD_OS2
-    SDL_OS2TLSFree(); /* thread/os2/SDL_systls.c */
-#endif
     SDL_OS2Quit();
 #endif
 
     /* Shut down requested initialized subsystems */
-#if !SDL_SENSOR_DISABLED
+#ifndef SDL_SENSOR_DISABLED
     if (flags & SDL_INIT_SENSOR) {
         if (SDL_PrivateShouldQuitSubsystem(SDL_INIT_SENSOR)) {
             SDL_SensorQuit();
@@ -368,29 +410,27 @@ void SDL_QuitSubSystem(Uint32 flags)
     }
 #endif
 
-#if !SDL_JOYSTICK_DISABLED
+#ifndef SDL_JOYSTICK_DISABLED
     if (flags & SDL_INIT_GAMECONTROLLER) {
-        /* game controller implies joystick */
-        flags |= SDL_INIT_JOYSTICK;
-
         if (SDL_PrivateShouldQuitSubsystem(SDL_INIT_GAMECONTROLLER)) {
             SDL_GameControllerQuit();
+            /* game controller implies joystick */
+            SDL_QuitSubSystem(SDL_INIT_JOYSTICK);
         }
         SDL_PrivateSubsystemRefCountDecr(SDL_INIT_GAMECONTROLLER);
     }
 
     if (flags & SDL_INIT_JOYSTICK) {
-        /* joystick implies events */
-        flags |= SDL_INIT_EVENTS;
-
         if (SDL_PrivateShouldQuitSubsystem(SDL_INIT_JOYSTICK)) {
             SDL_JoystickQuit();
+            /* joystick implies events */
+            SDL_QuitSubSystem(SDL_INIT_EVENTS);
         }
         SDL_PrivateSubsystemRefCountDecr(SDL_INIT_JOYSTICK);
     }
 #endif
 
-#if !SDL_HAPTIC_DISABLED
+#ifndef SDL_HAPTIC_DISABLED
     if (flags & SDL_INIT_HAPTIC) {
         if (SDL_PrivateShouldQuitSubsystem(SDL_INIT_HAPTIC)) {
             SDL_HapticQuit();
@@ -399,31 +439,29 @@ void SDL_QuitSubSystem(Uint32 flags)
     }
 #endif
 
-#if !SDL_AUDIO_DISABLED
+#ifndef SDL_AUDIO_DISABLED
     if (flags & SDL_INIT_AUDIO) {
-        /* audio implies events */
-        flags |= SDL_INIT_EVENTS;
-
         if (SDL_PrivateShouldQuitSubsystem(SDL_INIT_AUDIO)) {
             SDL_AudioQuit();
+            /* audio implies events */
+            SDL_QuitSubSystem(SDL_INIT_EVENTS);
         }
         SDL_PrivateSubsystemRefCountDecr(SDL_INIT_AUDIO);
     }
 #endif
 
-#if !SDL_VIDEO_DISABLED
+#ifndef SDL_VIDEO_DISABLED
     if (flags & SDL_INIT_VIDEO) {
-        /* video implies events */
-        flags |= SDL_INIT_EVENTS;
-
         if (SDL_PrivateShouldQuitSubsystem(SDL_INIT_VIDEO)) {
             SDL_VideoQuit();
+            /* video implies events */
+            SDL_QuitSubSystem(SDL_INIT_EVENTS);
         }
         SDL_PrivateSubsystemRefCountDecr(SDL_INIT_VIDEO);
     }
 #endif
 
-#if !SDL_TIMERS_DISABLED && !SDL_TIMER_DUMMY
+#if !defined(SDL_TIMERS_DISABLED) && !defined(SDL_TIMER_DUMMY)
     if (flags & SDL_INIT_TIMER) {
         if (SDL_PrivateShouldQuitSubsystem(SDL_INIT_TIMER)) {
             SDL_TimerQuit();
@@ -432,7 +470,7 @@ void SDL_QuitSubSystem(Uint32 flags)
     }
 #endif
 
-#if !SDL_EVENTS_DISABLED
+#ifndef SDL_EVENTS_DISABLED
     if (flags & SDL_INIT_EVENTS) {
         if (SDL_PrivateShouldQuitSubsystem(SDL_INIT_EVENTS)) {
             SDL_EventsQuit();
@@ -477,30 +515,24 @@ void SDL_Quit(void)
     SDL_bInMainQuit = SDL_TRUE;
 
     /* Quit all subsystems */
-#if SDL_VIDEO_DRIVER_WINDOWS
+#ifdef SDL_VIDEO_DRIVER_WINDOWS
     SDL_HelperWindowDestroy();
 #endif
     SDL_QuitSubSystem(SDL_INIT_EVERYTHING);
 
-#if !SDL_TIMERS_DISABLED
-    SDL_TicksQuit();
+#ifdef SDL_USE_LIBDBUS
+    SDL_DBus_Quit();
 #endif
 
     SDL_ClearHints();
     SDL_AssertionsQuit();
 
-#if SDL_USE_LIBDBUS
-    SDL_DBus_Quit();
-#endif
-
-    SDL_LogQuit();
-
     /* Now that every subsystem has been quit, we reset the subsystem refcount
      * and the list of initialized subsystems.
      */
     SDL_memset(SDL_SubsystemRefCount, 0x0, sizeof(SDL_SubsystemRefCount));
 
-    SDL_TLSCleanup();
+    SDL_QuitMainThread();
 
     SDL_bInMainQuit = SDL_FALSE;
 }
@@ -511,7 +543,7 @@ void SDL_GetVersion(SDL_version *ver)
     static SDL_bool check_hint = SDL_TRUE;
     static SDL_bool legacy_version = SDL_FALSE;
 
-    if (ver == NULL) {
+    if (!ver) {
         return;
     }
 
@@ -544,71 +576,71 @@ int SDL_GetRevisionNumber(void)
 /* Get the name of the platform */
 const char *SDL_GetPlatform(void)
 {
-#if __AIX__
+#if defined(__AIX__)
     return "AIX";
-#elif __ANDROID__
+#elif defined(__ANDROID__)
     return "Android";
-#elif __BSDI__
+#elif defined(__BSDI__)
     return "BSDI";
-#elif __DREAMCAST__
+#elif defined(__DREAMCAST__)
     return "Dreamcast";
-#elif __EMSCRIPTEN__
+#elif defined(__EMSCRIPTEN__)
     return "Emscripten";
-#elif __FREEBSD__
+#elif defined(__FREEBSD__)
     return "FreeBSD";
-#elif __HAIKU__
+#elif defined(__HAIKU__)
     return "Haiku";
-#elif __HPUX__
+#elif defined(__HPUX__)
     return "HP-UX";
-#elif __IRIX__
+#elif defined(__IRIX__)
     return "Irix";
-#elif __LINUX__
+#elif defined(__LINUX__)
     return "Linux";
-#elif __MINT__
+#elif defined(__MINT__)
     return "Atari MiNT";
-#elif __MACOS__
+#elif defined(__MACOS__)
     return "MacOS Classic";
-#elif __MACOSX__
+#elif defined(__MACOSX__)
     return "Mac OS X";
-#elif __NACL__
+#elif defined(__NACL__)
     return "NaCl";
-#elif __NETBSD__
+#elif defined(__NETBSD__)
     return "NetBSD";
-#elif __OPENBSD__
+#elif defined(__OPENBSD__)
     return "OpenBSD";
-#elif __OS2__
+#elif defined(__OS2__)
     return "OS/2";
-#elif __OSF__
+#elif defined(__OSF__)
     return "OSF/1";
-#elif __QNXNTO__
+#elif defined(__QNXNTO__)
     return "QNX Neutrino";
-#elif __RISCOS__
+#elif defined(__RISCOS__)
     return "RISC OS";
-#elif __SOLARIS__
+#elif defined(__SOLARIS__)
     return "Solaris";
-#elif __WIN32__
+#elif defined(__WIN32__)
     return "Windows";
-#elif __WINRT__
+#elif defined(__WINRT__)
     return "WinRT";
-#elif __WINGDK__
+#elif defined(__WINGDK__)
     return "WinGDK";
-#elif __XBOXONE__
+#elif defined(__XBOXONE__)
     return "Xbox One";
-#elif __XBOXSERIES__
+#elif defined(__XBOXSERIES__)
     return "Xbox Series X|S";
-#elif __TVOS__
+#elif defined(__TVOS__)
     return "tvOS";
-#elif __IPHONEOS__
+#elif defined(__IPHONEOS__)
     return "iOS";
-#elif __PS2__
+#elif defined(__PS2__)
     return "PlayStation 2";
-#elif __PSP__
+#elif defined(__PSP__)
     return "PlayStation Portable";
-#elif __VITA__
+#elif defined(__VITA__)
     return "PlayStation Vita";
-#elif __NGAGE__
+#elif defined(__NGAGE__)
     return "Nokia N-Gage";
-#elif __3DS__
+#elif defined(__3DS__)
     return "Nintendo 3DS";
 #else
     return "Unknown (see SDL_platform.h)";
@@ -617,10 +649,10 @@ const char *SDL_GetPlatform(void)
 
 SDL_bool SDL_IsTablet(void)
 {
-#if __ANDROID__
+#if defined(__ANDROID__)
     extern SDL_bool SDL_IsAndroidTablet(void);
     return SDL_IsAndroidTablet();
-#elif __IPHONEOS__
+#elif defined(__IPHONEOS__)
     extern SDL_bool SDL_IsIPad(void);
     return SDL_IsIPad();
 #else
