@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2014 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -20,11 +20,11 @@
 */
 #include "../../SDL_internal.h"
 
-#ifndef _SDL_nullvideo_h
-#define _SDL_nullvideo_h
+#ifndef SDL_nullvideo_h_
+#define SDL_nullvideo_h_
 
 #include "../SDL_sysvideo.h"
 
-#endif /* _SDL_nullvideo_h */
+#endif /* SDL_nullvideo_h_ */
 
 /* vi: set ts=4 sw=4 expandtab: */
